import nose.tools as nt
from mock import patch
from copy import deepcopy

import ckanext.odsh.collection.helpers as helpers_collection


class TestHelpersCollection(object):
    def setUp(self):
        self.datasets_in_collection = [
            {
                'name': 'dataset 1',
            },
            {
                'name': 'dataset 2',
            },
            {
                'name': 'dataset 3',
            },
        ]

        self.collection_dict = {
            'name': 'collection-name',
            'id': 'someid',
            'title': 'collection title',
            'relationships': [
                {
                    'object': 'dataset 1',
                },
                {
                    'object': 'dataset 2',
                },
                {
                    'object': 'dataset 3',
                },
            ]
        }

        self.patch_get_package_dict = patch.object(
            helpers_collection,
            'get_package_dict',
            return_value=self.collection_dict
        )
        self.patch_get_package_dict.start()

        self.patch_url_from_id = patch.object(
            helpers_collection,
            'url_from_id',
            new=lambda id: 'http://{}'.format(id)
        )
        self.patch_url_from_id.start()

        self.patch_url_last_member = patch.object(
            helpers_collection,
            'url_last_member',
            new=lambda collection_name: 'http://{}/last'.format(
                collection_name)
        )
        self.patch_url_last_member.start()

        self.patch_get_datasets_from_solr = patch.object(
            helpers_collection,
            'get_datasets_from_solr',
            return_value=self.datasets_in_collection
        )
        self.patch_get_datasets_from_solr.start()

    def tearDown(self):
        self.patch_get_package_dict.stop()
        self.patch_url_from_id.stop()
        self.patch_url_last_member.stop()
        self.patch_get_datasets_from_solr.stop()

    def test_patch_get_package_dict(self):
        collection_dict = helpers_collection.get_package_dict(
            'collection_name')
        nt.assert_equal(collection_dict, self.collection_dict)

    def test_patch_url_from_id(self):
        nt.assert_equal(helpers_collection.url_from_id(
            'some_id'), 'http://some_id')

    def test_patch_url_last_member(self):
        nt.assert_equal(helpers_collection.url_last_member(
            'some_collection'), 'http://some_collection/last')

    def test_get_collection_info_WITHOUT_dataset_dict(self):
        collection_info = helpers_collection.get_collection_info('some_id')
        collection_info_expected = {
            'url': 'http://collection-name',
            'first_member': {
                'name': 'dataset 1',
                'url': 'http://dataset 1'
            },
            'last_member': {'name': 'dataset 3', 'url': 'http://dataset 3'},
            'members': [{'name': 'dataset 1'}, {'name': 'dataset 2'}, {'name': 'dataset 3'}],
            'persistent_link_last_member': 'http://collection-name/last',
            'predecessor': {'url': None},
            'successor': {'url': None},
            'title': 'collection title'
        }
        nt.assert_equal(collection_info, collection_info_expected)
    
    def test_get_collection_info_WITH_dataset_dict(self):
        collection_info = helpers_collection.get_collection_info('some_id', {'name': 'dataset 2'})
        collection_info_expected = {
            'url': 'http://collection-name',
            'first_member': {'name': 'dataset 1', 'url': 'http://dataset 1'},
            'last_member': {'name': 'dataset 3', 'url': 'http://dataset 3'},
            'members': [{'name': 'dataset 1'}, {'name': 'dataset 2'}, {'name': 'dataset 3'}],
            'persistent_link_last_member': 'http://collection-name/last',
            'predecessor': {'url': 'http://dataset 1'},
            'successor': {'url': 'http://dataset 3'},
            'title': 'collection title'
        }
        nt.assert_equal(collection_info, collection_info_expected)

    def test_get_collection_id_if_in_collection(self):
        dataset_dict = {
            'relationships_as_subject': [
                {'__extras': {
                    'object_package_id': 'some_id'
                }}
            ]
        }
        collection_id = helpers_collection.get_collection_id(dataset_dict)
        nt.assert_equal(collection_id, 'some_id')
    
    def test_get_collection_id_if_in_collection_alternative_version(self):
        dataset_dict = {
            'relationships': [
                {'object': 'some_id'}
            ]
        }
        collection_id = helpers_collection.get_collection_id(dataset_dict)
        nt.assert_equal(collection_id, 'some_id')

    def test_get_collection_id_if_NOT_in_collection(self):
        dataset_dict = {
            'relationships_as_subject': []
        }
        collection_id = helpers_collection.get_collection_id(dataset_dict)
        nt.assert_equal(collection_id, None)

    def test_get_dataset_names(self):
        dataset_names = helpers_collection.get_dataset_names(dict())
        nt.assert_equal(dataset_names, ['dataset 1', 'dataset 2', 'dataset 3'])

    def test_gather_collection_info_without_reference_package(self):
        collection_info = helpers_collection.gather_collection_info(
            self.collection_dict, self.datasets_in_collection, dataset_dict=None)
        collection_info_expected = {
            'url': 'http://collection-name',
            'first_member': {'name': 'dataset 1', 'url': 'http://dataset 1'},
            'last_member': {'name': 'dataset 3', 'url': 'http://dataset 3'},
            'members': [
                {'name': 'dataset 1'},
                {'name': 'dataset 2'},
                {'name': 'dataset 3'}],
            'persistent_link_last_member': 'http://collection-name/last',
            'predecessor': {'url': None},
            'successor': {'url': None},
            'title': 'collection title'
        }
        nt.assert_equal(collection_info, collection_info_expected)

    def test_gather_collection_info_with_reference_package_1(self):
        collection_info = helpers_collection.gather_collection_info(
            self.collection_dict, self.datasets_in_collection, dataset_dict={'name': 'dataset 1'})
        collection_info_expected = {
            'url': 'http://collection-name',
            'first_member': {'name': 'dataset 1', 'url': 'http://dataset 1'},
            'last_member': {'name': 'dataset 3', 'url': 'http://dataset 3'},
            'members': [
                {'name': 'dataset 1'},
                {'name': 'dataset 2'},
                {'name': 'dataset 3'}],
            'persistent_link_last_member': 'http://collection-name/last',
            'predecessor': {'url': None},
            'successor': {'url': 'http://dataset 2'},
            'title': 'collection title'
        }
        nt.assert_equal(collection_info, collection_info_expected)

    def test_gather_collection_info_with_reference_package_2(self):
        collection_info = helpers_collection.gather_collection_info(
            self.collection_dict, self.datasets_in_collection, dataset_dict={'name': 'dataset 2'})
        collection_info_expected = {
            'url': 'http://collection-name',
            'first_member': {'name': 'dataset 1', 'url': 'http://dataset 1'},
            'last_member': {'name': 'dataset 3', 'url': 'http://dataset 3'},
            'members': [
                {'name': 'dataset 1'},
                {'name': 'dataset 2'},
                {'name': 'dataset 3'}],
            'persistent_link_last_member': 'http://collection-name/last',
            'predecessor': {'url': 'http://dataset 1'},
            'successor': {'url': 'http://dataset 3'},
            'title': 'collection title'
        }
        nt.assert_equal(collection_info, collection_info_expected)

    def test_gather_collection_info_with_reference_package_3(self):
        collection_info = helpers_collection.gather_collection_info(
            self.collection_dict, self.datasets_in_collection, dataset_dict={'name': 'dataset 3'})
        collection_info_expected = {
            'url': 'http://collection-name',
            'first_member': {'name': 'dataset 1', 'url': 'http://dataset 1'},
            'last_member': {'name': 'dataset 3', 'url': 'http://dataset 3'},
            'members': [
                {'name': 'dataset 1'},
                {'name': 'dataset 2'},
                {'name': 'dataset 3'}],
            'persistent_link_last_member': 'http://collection-name/last',
            'predecessor': {'url': 'http://dataset 2'},
            'successor': {'url': None},
            'title': 'collection title'
        }
        nt.assert_equal(collection_info, collection_info_expected)
